@@ -14,7 +14,8 @@ module Fog
                  :openstack_project_name, :openstack_project_id,
                  :openstack_project_domain, :openstack_user_domain, :openstack_domain_name,
                  :openstack_project_domain_id, :openstack_user_domain_id, :openstack_domain_id,
-                 :openstack_identity_api_version
+                 :openstack_identity_api_version, :openstack_application_credential_id,
+                 :openstack_application_credential_secret
 
       ## MODELS
       #
