@@ -16,6 +16,8 @@ module Fog
       attr_reader :openstack_project_id
       attr_reader :openstack_project_domain_id
       attr_reader :openstack_identity_api_version
+      attr_reader :openstack_application_credential_id
+      attr_reader :openstack_application_credential_secret
 
       # fallback
       def self.not_found_class
@@ -50,17 +52,18 @@ module Fog
       def request(params, parse_json = true)
         retried = false
         begin
+          authenticate! if @expires && (@expires - Time.now.utc).to_i < 60
+
           response = @connection.request(
             params.merge(
-              :headers => headers(params.delete(:headers)),
+              :headers => headers(params[:headers]),
               :path    => "#{@path}/#{params[:path]}"
             )
           )
-        rescue Excon::Errors::Unauthorized => error
+        rescue Excon::Errors::Unauthorized, Excon::Error::Unauthorized => error
           # token expiration and token renewal possible
           if error.response.body != 'Bad username or password' && @openstack_can_reauthenticate && !retried
-            @openstack_must_reauthenticate = true
-            authenticate
+            authenticate!
             retried = true
             retry
           # bad credentials or token renewal not possible
@@ -171,6 +174,9 @@ module Fog
           instance_variable_set "@#{openstack_param}".to_sym, value
         end
 
+        # Ensure OpenStack User's Password is always a String
+        @openstack_api_key = @openstack_api_key.to_s if @openstack_api_key
+
         @auth_token ||= options[:openstack_auth_token]
         @openstack_must_reauthenticate = false
         @openstack_endpoint_type = options[:openstack_endpoint_type] || 'public'
@@ -180,10 +186,11 @@ module Fog
           @openstack_can_reauthenticate = false
         else
           missing_credentials = []
-
-          missing_credentials << :openstack_api_key unless @openstack_api_key
-          unless @openstack_username || @openstack_userid
-            missing_credentials << 'openstack_username or openstack_userid'
+          unless @openstack_application_credential_secret and @openstack_application_credential_id
+            missing_credentials << :openstack_api_key unless @openstack_api_key
+            unless @openstack_username || @openstack_userid
+              missing_credentials << 'openstack_username/openstack_userid or openstack_application_credential_secret and openstack_application_credential_id'
+            end
           end
           unless missing_credentials.empty?
             raise ArgumentError, "Missing required arguments: #{missing_credentials.join(', ')}"
@@ -197,7 +204,7 @@ module Fog
 
         @openstack_service_type = options[:openstack_service_type] || default_service_type
         @openstack_endpoint_type = options[:openstack_endpoint_type] || default_endpoint_type
-        @openstack_endpoint_type.gsub!(/URL/, '')
+        @openstack_endpoint_type = @openstack_endpoint_type.gsub(/URL/, '')
         @connection_options = options[:connection_options] || {}
         @persistent = options[:persistent] || false
       end
@@ -221,7 +228,7 @@ module Fog
           @current_user = token.user['name']
           @current_user_id          = token.user['id']
           @current_tenant           = token.tenant
-          @expires                  = token.expires
+          @expires                  = Time.parse(token.expires)
           @auth_token               = token.token
           @unscoped_token           = token.token
           @openstack_must_reauthenticate = false
@@ -237,6 +244,11 @@ module Fog
 
         true
       end
+
+      def authenticate!
+        @openstack_must_reauthenticate = true
+        authenticate
+      end
     end
   end
 end
